// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KITINERARY_VERSION_H
#define KITINERARY_VERSION_H

#define KITINERARY_VERSION_STRING "6.7.90"
#define KITINERARY_VERSION_MAJOR 6
#define KITINERARY_VERSION_MINOR 7
#define KITINERARY_VERSION_PATCH 90
#define KITINERARY_VERSION ((6<<16)|(7<<8)|(90))

#endif
